﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Net;
using System.Windows;
using System.Xml.Linq;
using EventTracker.SLDashboard.EventTracker.OService;
using GalaSoft.MvvmLight.Messaging;

namespace EventTracker.SLDashboard
{
    public class StateManager
    {
        public ObservableCollection<Conference> Conferences;
        public ObservableCollection<Track> Tracks;
        public ObservableCollection<Session> Sessions;

        public Conference CurrentConference;
        public Track CurrentTrack;

        readonly string baseUri = "http://huwebsrv01/EventTracker/services/eventtrackerservice.svc/";

        public StateManager()
        {
            Conferences = new ObservableCollection<Conference>();
            Tracks = new ObservableCollection<Track>();
            Sessions = new ObservableCollection<Session>();

            CurrentConference = null;
            CurrentTrack = null;

            SetSubscriptions();
        }

        private void SetSubscriptions()
        {
            Messenger.Default.Register<string>(this, "GetConferences", GetConferences);

            Messenger.Default.Register<string>(this, "GetTracks", GetConferenceTracks);

            Messenger.Default.Register<Conference>(this, "SetCurrentConference", SetCurrentConference);

            Messenger.Default.Register<Track>(this, "SetCurrentTrack", SetCurrentTrack);

            Messenger.Default.Register<string>(this, "GoSessionViewLoaded", GoLoadSessions);
        }

        public void SetCurrentConference(Conference conf)
        {
            CurrentConference = conf;
        }

        public void SetCurrentTrack(Track track)
        {
            CurrentTrack = track;
        }

        #region Conferences

        public void GetConferences(string obj)
        {
            var client = new WebClient();

            Uri thisUri = new Uri(baseUri + "Conferences", UriKind.Absolute);

            client.DownloadStringCompleted += (ss, ee) =>
            {
                if (ee.Result != null)
                {
                    TextReader reader = null;

                    try
                    {
                        reader = new StringReader(ee.Result);

                        MakeConferences(XElement.Load(reader));
                    }
                    catch (Exception eee) { Debug.WriteLine(eee.Message + "/n" + eee.StackTrace); }
                    finally { reader.Dispose(); }
                }
                else
                {
                    MessageBox.Show(ee.Error.Message);
                }
            };

            client.DownloadStringAsync(thisUri);
        }

        void MakeConferences(XElement xe)
        {
            Conferences.Clear();

            IEnumerable<XElement> elems = from x in xe.Elements()
                                          where x.Name.LocalName == "entry"
                                          select x;

            foreach (XElement elem in elems)
            {
                XElement props = elem.GetContentProps();

                Conference cf = MakeConference(props);

                Conferences.Add(cf);
            }

            Messenger.Default.Send<ObservableCollection<Conference>>(Conferences, "ConferencesLoaded");
        }

        private Conference MakeConference(XElement props)
        {
            Conference cf = new Conference();

            cf.ID = props.GetLocalNameIntProperty("ID");
            cf.Name = props.GetLocalNameProperty("Name");
            cf.StartDate = props.GetLocalNameDateTimeProperty("StartDate");
            cf.EndDate = props.GetLocalNameDateTimeProperty("EndDate");
            cf.Capacity = props.GetLocalNameIntProperty("Capacity");
            cf.Description = props.GetLocalNameProperty("Description");
            cf.Location = props.GetLocalNameProperty("Location");
            cf.Website = props.GetLocalNameProperty("Website");

            return cf;
        }

        #endregion

        #region Tracks

        public void GetConferenceTracks(string obj)
        {
            Uri thisUri = new Uri(baseUri + "Conferences(" + CurrentConference.ID.ToString() + ")/Tracks", UriKind.Absolute);

            var client = new WebClient();

            client.DownloadStringCompleted += ((ss, ee) =>
            {
                if (ee.Result != null)
                {
                    TextReader reader = null;

                    try
                    {
                        reader = new StringReader(ee.Result);

                        MakeTracks(XElement.Load(reader));
                    }
                    catch (Exception eee) { Debug.WriteLine(eee.Message + "/n" + eee.InnerException); }
                    finally { reader.Dispose(); }
                }
            });

            client.DownloadStringAsync(thisUri);
        }

        void MakeTracks(XElement xe)
        {
            Tracks.Clear();

            IEnumerable<XElement> tracksXML = from x in xe.Elements()
                                              where x.Name.LocalName == "entry"
                                              select x;

            foreach (XElement trackX in tracksXML)
            {
                XElement props = trackX.GetContentProps();

                Track track = MakeTrack(props);

                Tracks.Add(track);
            }

            Messenger.Default.Send<ObservableCollection<Track>>(Tracks, "TracksLoaded");
        }

        private Track MakeTrack(XElement props)
        {
            Track track = new Track();

            track.Title = props.GetLocalNameProperty("Title");
            track.ID = props.GetLocalNameIntProperty("ID");
            track.Description = props.GetLocalNameProperty("Description");

            return track;
        }

        #endregion

        #region Sessions

        public void GoLoadSessions(string obj)
        {
            SendConferenceTrackInfo();

            var client = new WebClient();

            Uri thisUri = new Uri(baseUri + "Conferences(" + CurrentConference.ID.ToString() + ")/Tracks(" + CurrentTrack.ID.ToString() + ")/Sessions?$expand=Speaker/Person", UriKind.Absolute);

            client.DownloadStringCompleted += ((ss, ee) =>
            {
                if (ee.Result != null)
                {
                    TextReader reader = null;

                    try
                    {
                        reader = new StringReader(ee.Result);

                        MakeSessions(XElement.Load(reader));
                    }
                    catch (Exception eee) { Debug.WriteLine(eee.Message + "/n" + eee.InnerException); }
                    finally { reader.Dispose(); }
                }
            });

            client.DownloadStringAsync(thisUri);
        }

        void SendConferenceTrackInfo()
        {
            // send our Conference Name and Track Title

            string NameTitle = CurrentConference.Name + "|" + CurrentTrack.Title;

            Messenger.Default.Send<string>(NameTitle, "SessionConferenceTrack");
        }

        void MakeSessions(XElement xe)
        {
            Sessions.Clear();

            IEnumerable<XElement> sessionXML = from x in xe.Elements()
                                               where x.Name.LocalName == "entry"
                                               select x;

            foreach (XElement sessionX in sessionXML)
            {
                XElement props = sessionX.GetContentProps();

                Session session = MakeSession(props);

                Speaker spk = new Speaker();

                XName xn = XName.Get("title");

                XElement speakInfo = sessionX.GetAttributeElement("link", xn, "Speaker")
                    .GetInlineEntries()
                    .GetAttributeElement("link", xn, "Person")
                    .GetInlineEntries()
                    .GetContentProps();

                spk.Person = new Person();
                
                spk.Person.FirstName = speakInfo.GetLocalNameProperty("FirstName");
                spk.Person.LastName = speakInfo.GetLocalNameProperty("LastName");

                session.Speaker = spk;

                Sessions.Add(session);
            }

            Messenger.Default.Send<ObservableCollection<Session>>(Sessions, "SendSessionsList");
        }

        private Session MakeSession(XElement props)
        {
            Session session = new Session();
            
            session.ID = props.GetLocalNameIntProperty("ID");
            session.Title = props.GetLocalNameProperty("Title");
            session.Description = props.GetLocalNameProperty("Description");
            session.StartTime = props.GetLocalNameDateTimeProperty("StartTime");
            session.EndTime = props.GetLocalNameDateTimeProperty("EndTime");
            session.SkillLevel = props.GetLocalNameIntProperty("SkillLevel");

            return session;
        }

        #endregion
    }
}
