﻿using System;
using System.Linq;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using EventTracker.DAL;
using EventTracker.Infrastructure;
using EventTracker.Models;
using EventTracker.Web.Models;

namespace EventTracker.Controllers
{
    [HandleError]
    public class AccountController : Controller
    {
        public IFormsAuthenticationService FormsService { get; set; }
        public IMembershipService MembershipService { get; set; }
        private readonly IContextManager _contextMgr;

        public AccountController(ContextManager contextMgr)
        {
            _contextMgr = contextMgr;
        }

        protected override void Initialize(RequestContext requestContext)
        {
            if (FormsService == null) { FormsService = new FormsAuthenticationService(); }
            if (MembershipService == null) { MembershipService = new AccountMembershipService(); }

            base.Initialize(requestContext);
        }

        [Authorize]
        public ActionResult Dashboard()
        {
            var model = new DashboardViewModel();

            model.OrganizerFor = _contextMgr.Context.GetOrganizedEventsForUser(User.Identity.Name).ToList();
            model.Sessions = _contextMgr.Context.GetUserSessions(User.Identity.Name).ToList();
            //model.Attended = _contextMgr.Context.GetAttendedEventsForuser(User.Identity.Name).ToList();
            //model.Attending = _contextMgr.Context.GetAttendingEventsForuser(User.Identity.Name).ToList();

            Person person = _contextMgr.PersonRepository.GetByUsername(User.Identity.Name);
            if (person.IsAttendee)
            {
                model.Attended = _contextMgr.ConferenceRepository
                                           .Select()
                                           .Where(o => o.Organizers.Any(x => x.PersonID.HasValue && x.PersonID.Value == person.ID) && o.EndDate < DateTime.Now)
                                           .ToList();


                model.Attending = _contextMgr.ConferenceRepository
                                            .Select()
                                            .Where(o => o.Attendees.Any(x => x.PersonID.HasValue && x.PersonID.Value == person.ID) && o.EndDate >= DateTime.Now)
                                            .ToList();

            }

            //if (person.IsOrganizer)
            //{
            //    model.OrganizerFor = _contextMgr.ConferenceRepository
            //                                    .Select()
            //                                    .Where(o => o.Organizers.Any(x => x.PersonID.HasValue && x.PersonID.Value == person.ID))
            //                                    .ToList();

            //}

            //if (person.IsSpeaker)
            //{
            //    model.Sessions = _contextMgr.SessionRepository
            //                                .Select()
            //                                .Where(o => o.SpeakerID == person.ID)
            //                                .ToList();

            //}

            return View(model);
        }

        // **************************************
        // URL: /Account/LogOn
        // **************************************

        public ActionResult LogOn()
        {
            return View();
        }

        [HttpPost]
        public ActionResult LogOn(LogOnModel model, string returnUrl)
        {
            if (ModelState.IsValid)
            {
                if (MembershipService.ValidateUser(model.UserName, model.Password))
                {
                    FormsService.SignIn(model.UserName, model.RememberMe);
                    if (!String.IsNullOrEmpty(returnUrl))
                    {
                        return Redirect(returnUrl);
                    }
                    else
                    {
                        return RedirectToAction("Index", "Home");
                    }
                }
                else
                {
                    ModelState.AddModelError("", "The user name or password provided is incorrect.");
                }
            }

            // If we got this far, something failed, redisplay form
            return View(model);
        }

        // **************************************
        // URL: /Account/LogOff
        // **************************************

        public ActionResult LogOff()
        {
            FormsService.SignOut();

            return RedirectToAction("Index", "Home");
        }

        // **************************************
        // URL: /Account/Register
        // **************************************

        public ActionResult Register(AccountRegistrationType accountType)
        {
            ViewData["PasswordLength"] = MembershipService.MinPasswordLength;
            ViewData["AccountType"] = accountType;
            return View();
        }

        [HttpPost]
        public ActionResult Register(RegisterModel model)
        {
            if (ModelState.IsValid)
            {
                // Attempt to register the user
                MembershipCreateStatus createStatus = MembershipService.CreateUser(model.UserName, model.Password, model.Email);

                if (createStatus == MembershipCreateStatus.Success)
                {
                    // Create a person associated with this user
                    _contextMgr.PersonRepository.Add(new DAL.Person()
                    {
                        Username = model.UserName,
                        FirstName = model.FirstName,
                        LastName = model.LastName,
                        IsOrganizer = model.RegistrationType == AccountRegistrationType.Organizer,
                        IsSpeaker = model.RegistrationType == AccountRegistrationType.Speaker,
                        //IsAttendee = model.RegistrationType == AccountRegistrationType.Attendee
                        IsAttendee = true
                    });
                    _contextMgr.Context.SaveChanges();

                    // Set role of user
                    if (model.RegistrationType == AccountRegistrationType.Organizer) Roles.AddUserToRole(model.UserName, "Organizer");
                    if (model.RegistrationType == AccountRegistrationType.Speaker) Roles.AddUserToRole(model.UserName, "Speaker");

                    // Users are always entered into the Attendee role
                    //if (model.RegistrationType == AccountRegistrationType.Attendee) Roles.AddUserToRole(model.UserName, "Attendee");
                    Roles.AddUserToRole(model.UserName, "Attendee");


                    FormsService.SignIn(model.UserName, false /* createPersistentCookie */);
                    return RedirectToAction("Index", "Home");
                }
                else
                {
                    ModelState.AddModelError("", AccountValidation.ErrorCodeToString(createStatus));
                }
            }

            // If we got this far, something failed, redisplay form
            ViewData["PasswordLength"] = MembershipService.MinPasswordLength;
            ViewBag.AccountType = model.RegistrationType;
            return View(model);
        }

        // **************************************
        // URL: /Account/ChangePassword
        // **************************************

        [Authorize]
        public ActionResult ChangePassword()
        {
            ViewData["PasswordLength"] = MembershipService.MinPasswordLength;
            return View();
        }

        [Authorize]
        [HttpPost]
        public ActionResult ChangePassword(ChangePasswordModel model)
        {
            if (ModelState.IsValid)
            {
                if (MembershipService.ChangePassword(User.Identity.Name, model.OldPassword, model.NewPassword))
                {
                    return RedirectToAction("ChangePasswordSuccess");
                }
                else
                {
                    ModelState.AddModelError("", "The current password is incorrect or the new password is invalid.");
                }
            }

            // If we got this far, something failed, redisplay form
            ViewData["PasswordLength"] = MembershipService.MinPasswordLength;
            return View(model);
        }

        // **************************************
        // URL: /Account/ChangePasswordSuccess
        // **************************************

        public ActionResult ChangePasswordSuccess()
        {
            return View();
        }

    }
}
