﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Web.Mvc;
using AutoMapper;
using EventTracker.DAL;
using EventTracker.Helpers;
using EventTracker.Infrastructure;
using EventTracker.Models;

namespace EventTracker.Controllers
{
    public class ConferencesController : Controller
    {
        private readonly IContextManager _contextMgr;

        public ConferencesController(IContextManager contextMgr)
        {
            _contextMgr = contextMgr;
        }

        public ActionResult Index(ConferenceFilter? filter, FormCollection collection)
        {
            ConferenceFilter vFilter = filter ?? ConferenceFilter.Attendee;

            List<ConferenceModel> model = null;

            if (vFilter == ConferenceFilter.Attendee)
            {
                model = _contextMgr.ConferenceRepository.Select()
                                                        .ToList()
                                                        .Select(c => Mapper.Map<Conference, ConferenceModel>(c))
                                                        .ToList();
            }
            else if (vFilter == ConferenceFilter.Organizer)
            {
                Person person = _contextMgr.PersonRepository.GetByUsername(User.Identity.Name);
                model = _contextMgr.OrganizerRepository.Select()
                        .Where(o => o.Person == person)
                        .ToList()
                        .Select(c => Mapper.Map<Conference, ConferenceModel>(c.Conference))
                        .ToList();
            }
            else
            {
                model = new List<ConferenceModel>();
            }

            ViewBag.IsOrganizer = User.IsOrganizer();

            return View(model);
        }

        [Authorize(Roles = "Organizer")]
        public ActionResult Create()
        {
            return View(new ConferenceModel());
        }

        [HttpPost]
        [Authorize(Roles = "Organizer")]
        public ActionResult Create(ConferenceModel model, FormCollection collection)
        {
            try
            {
                // Create a conference
                var conference = Mapper.Map<ConferenceModel, Conference>(model);
                _contextMgr.ConferenceRepository.Add(conference);

                // Create and associate and organizer account w/this conference
                Person person = _contextMgr.PersonRepository.GetByUsername(User.Identity.Name);
                conference.Organizers.Add(new Organizer()
                {
                    Person = person,
                    Conference = conference
                });

                _contextMgr.Context.SaveChanges();

                return RedirectToAction("Details", new { Id = conference.ID });
            }
            catch (Exception ex)
            {
                return View(ex);
            }
        }

        public ActionResult Details(int id)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();
            if (conference == null) return View("Error", "Conference was not found.");

            // Setup Model & ViewData
            ConferenceModel model = Mapper.Map<Conference, ConferenceModel>(conference);
            ViewData["IsOrganizer"] = User.IsOrganizerOfConference(conference);

            return View(model);
        }

        [Authorize(Roles = "Organizer")]
        public ActionResult Edit(int id)
        {
            Conference conference = _contextMgr.ConferenceRepository
                .Select()
                .Where(c => c.ID == id)
                .FirstOrDefault();

            if (conference == null) return View("Error", "Conference was not found.");

            var model = Mapper.Map<Conference, ConferenceModel>(conference);

            return View(model);
        }

        [HttpPost]
        [Authorize(Roles = "Organizer")]
        public ActionResult Edit(ConferenceModel model, FormCollection collection)
        {
            try
            {
                Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == model.Id).FirstOrDefault();
                if (conference == null) return View("Error", "Conference not found.");

                Mapper.Map<ConferenceModel, Conference>(model, conference);

                _contextMgr.Context.SaveChanges();

                return RedirectToAction("Details", new { Id = conference.ID });
            }
            catch
            {
                return View();
            }
        }

        [Authorize(Roles = "Organizer")]
        public ActionResult Delete(int id)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();
            if (conference == null) return View("Error", "Conference not found.");

            // Delete the organizers
            //foreach (Organizer organizer in conference.Organizers)
            //    _contextMgr.OrganizerRepository.Remove(organizer);

            // Delete the conference
            _contextMgr.ConferenceRepository.Remove(conference);

            // Save changes
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Index", new { filter = ConferenceFilter.Organizer });
        }

        public ActionResult Schedule(int id, string format)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();
            if (conference == null) return View("Error", "Conference not found.");

            EventTracker.ReportLibrary.EventSchedule report = new ReportLibrary.EventSchedule();
            report.ReportParameters["ConferenceID"].Value = id;

            Telerik.Reporting.Processing.ReportProcessor reportProcessor = new Telerik.Reporting.Processing.ReportProcessor();
            Telerik.Reporting.Processing.RenderingResult renderingResult = reportProcessor.RenderReport(format, report, null);

            MemoryStream ms = new MemoryStream();
            ms.Write(renderingResult.DocumentBytes, 0, renderingResult.DocumentBytes.Length);
            ms.Flush();

            FileContentResult result = new FileContentResult(ms.GetBuffer(), renderingResult.MimeType);
            result.FileDownloadName = conference.Name + " Schedule." + format.ToLower();

            return result;
        }

        [HttpGet]
        public FileResult ScheduleStream(int id, string format)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().FirstOrDefault(c => c.ID == id);
            var documentName = conference.Name + " Schedule." + format.ToLower();
            EventTracker.ReportLibrary.EventSchedule report = new ReportLibrary.EventSchedule();
            report.ReportParameters["ConferenceID"].Value = id;

            Telerik.Reporting.Processing.ReportProcessor reportProcessor = new Telerik.Reporting.Processing.ReportProcessor();
            Telerik.Reporting.Processing.RenderingResult renderingResult = reportProcessor.RenderReport(format, report, null);

            MemoryStream ms = new MemoryStream();
            ms.Write(renderingResult.DocumentBytes, 0, renderingResult.DocumentBytes.Length);
            ms.Flush();

            ms.Position = 0;

            return new FileStreamResult(ms, "application/pdf")
                       {
                           FileDownloadName = documentName
                       };
        }

        public ActionResult Search()
        {
            return View();
        }

        public ActionResult Favorites()
        {
            return View();
        }

        [Authorize(Roles = RoleConstants.Organizer)]
        public ActionResult Organizers(int id)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();
            if (conference == null) return View("Error", "Conference not found.");

            var organizers = conference.Organizers.Select(o => new OrganizerModel() { Id = o.ID, FullName = o.Person.FirstName + " " + o.Person.LastName }).ToList();

            ViewBag.IsOrganizer = User.IsOrganizerOfConference(conference);
            ViewBag.ConferenceName = conference.Name;
            ViewBag.ConferenceId = conference.ID;

            return View(organizers);
        }

        [Authorize(Roles = RoleConstants.Organizer)]
        public ActionResult AddOrganizer(int id)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();

            if (conference == null) return View("Error", "Conference was not found.");

            // Determine if user is allowed to edit this conference
            if (!User.IsOrganizerOfConference(conference))
                return View("Error", "You are not allowed to edit this conference.");

            ViewBag.ConferenceId = conference.ID;

            return View();
        }

        [Authorize(Roles = RoleConstants.Organizer)]
        [HttpPost]
        public ActionResult AddOrganizer(int id, int personId, FormCollection collection)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();
            if (conference == null) return View("Error", "Conference was not found.");

            if (conference.Organizers.Where(o => o.PersonID == personId).FirstOrDefault() != null)
                return RedirectToAction("Organizers", new { Id = id });

            // Create and associate and organizer account w/this conference
            Person person = _contextMgr.PersonRepository.Select().Where(p => p.ID == personId).FirstOrDefault();
            if (conference == null) return View("Error", "Person was not found.");

            conference.Organizers.Add(new Organizer()
            {
                Person = person,
                Conference = conference
            });

            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Organizers", new { Id = id });
        }

        public ActionResult RemoveOrganizer(int id, int organizerId)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == id).FirstOrDefault();
            if (conference == null) return View("Error", "Conference was not found.");

            Organizer organizer = conference.Organizers.Where(o => o.ID == organizerId).FirstOrDefault();
            if (organizer == null) return View("Error", "Organizer was not found.");

            _contextMgr.OrganizerRepository.Remove(organizer);
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Organizers", new { Id = id });
        }
    }
}

