﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using AutoMapper;
using EventTracker.DAL;
using EventTracker.Helpers;
using EventTracker.Infrastructure;
using EventTracker.Models;

namespace EventTracker.Controllers
{
    public class SessionsController : Controller
    {
        readonly IContextManager _contextMgr;

        public SessionsController(IContextManager contextMgr)
        {
            _contextMgr = contextMgr;
        }

        //
        // GET: /Sessions/
        public ActionResult Index(int? trackId)
        {
            IEnumerable<SessionModel> model;

            if (!trackId.HasValue)
            {
                // Query for ALL sessions from ALL conferences
                model = _contextMgr.SessionRepository.Select()
                                   .ToList().Select(session => Mapper.Map<Session, SessionModel>(session));

                ViewBag.ConferenceId = -1;
                ViewBag.IsOrganizer = false;
                ViewBag.ConferenceName = "All Conferences";
                ViewBag.Title = "All";
            }
            else
            {
                // Query for sessions from specified conference ID
                //var conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == conferenceId.Value);
                var track = _contextMgr.TrackRepository.Select().Where(t => t.ID == trackId.Value).FirstOrDefault();

                model = _contextMgr.SessionRepository.Select()
                                          .Where(s => s.Track == track)
                                          .ToList()
                                          .Select(session => Mapper.Map<Session, SessionModel>(session));

                ViewBag.ConferenceId = track.Conference.ID;
                ViewBag.TrackId = trackId.Value;
                ViewBag.IsOrganizer = User.IsOrganizerOfConference(track.Conference);
                ViewBag.ConferenceName = track.Conference.Name;
                ViewBag.Title = track.Title;
            }

            return View(model);
        }

        public ActionResult _PartialListing(int trackId)
        {
            // return tracks specific to the specified conference
            Track track = _contextMgr.TrackRepository.Select()
                .Where(c => c.ID == trackId).FirstOrDefault();

            if (track == null) return View("PartialError", "Track was not found.");

            IEnumerable<SessionModel> model = _contextMgr.SessionRepository.Select()
                    .Where(t => t.Track == track)
                    .OrderBy(t => t.StartTime)
                    .ToList()
                    .Select(session => Mapper.Map<Session, SessionModel>(session));

            return View(model);
        }

        //
        // GET: /Sessions/Details/5
        public ActionResult Details(int id)
        {
            var session = _contextMgr.SessionRepository.Select().Where(s => s.ID == id).FirstOrDefault();
            if (session == null) return View("Error", "Session was not found.");

            ViewData["IsOrganizer"] = User.IsOrganizerOfConference(session.Track.Conference);

            var model = Mapper.Map<Session, SessionModel>(session);

            return View(model);
        }

        //
        // GET: /Sessions/Create
        [Authorize(Roles = RoleConstants.Organizer)]
        public ActionResult Create(int trackId)
        {
            // return tracks specific to the specified conference
            Track track = _contextMgr.TrackRepository.Select()
                .Where(c => c.ID == trackId).FirstOrDefault();

            if (track == null) return View("Error", "Track was not found.");

            // Determine if user is allowed to edit this conference
            if (!User.IsOrganizerOfConference(track.Conference))
                return View("Error", "You are not allowed to edit this conference.");

            return View(new SessionModel());
        }

        //
        // POST: /Sessions/Create

        [HttpPost]
        [Authorize(Roles = RoleConstants.Organizer)]
        public ActionResult Create(int trackId, SessionModel sessionModel, FormCollection collection)
        {
            Track track = _contextMgr.TrackRepository.Select().Where(c => c.ID == trackId).FirstOrDefault();
            if (track == null) return View("Error", "Conference not found.");

            Session session = Mapper.Map<SessionModel, Session>(sessionModel);
            
            // Create a new speaker association with this session
            Person person = _contextMgr.PersonRepository.GetByID(sessionModel.Speaker);
            session.Speaker = new Speaker()
            {
                Person = person,
                HasCheckedIn = false,
            };

            _contextMgr.SessionRepository.Add(session);
            _contextMgr.Context.SaveChanges();

            session.Track = track;
            _contextMgr.Context.SaveChanges();


            //session.Track = track;
            //track.Sessions.Add(session);

            //track.Sessions.Add(new Session()
            //{
            //    Title = sessionModel.Title,
            //    SkillLevel = sessionModel.SkillLevel,
            //    StartTime = sessionModel.StartTime,
            //    EndTime = sessionModel.EndTime,
            //    Track = track,
            //    Description = sessionModel.Description,
            //    Speaker = new Speaker()
            //    {
            //        HasCheckedIn = false,
            //        Person = _contextMgr.PersonRepository.GetByUsername(User.Identity.Name)
            //    }
            //});
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Index", new { trackId = trackId });
        }


        //
        // GET: /Sessions/Edit/5
        [Authorize(Roles = RoleConstants.Organizer)]
        public ActionResult Edit(int id)
        {
            var session = _contextMgr.SessionRepository.Select().Where(s => s.ID == id).FirstOrDefault();
            if (session == null) return View("Error", "Conference not found.");
            
            var model = Mapper.Map<Session, SessionModel>(session);

            return View(model);
        }

        //
        // POST: /Sessions/Edit/5

        [HttpPost]
        [Authorize(Roles = RoleConstants.Organizer)]
        public ActionResult Edit(SessionModel model, FormCollection collection)
        {
            Session session = _contextMgr.SessionRepository.Select().Where(t => t.ID == model.Id).FirstOrDefault();
            if (session == null) return View("Error", "Track was not found");

            Mapper.Map<SessionModel, Session>(model, session);
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Details", new { Id = session.ID });
        }

        [Authorize(Roles = "Organizer")]
        public ActionResult Delete(int id)
        {
            Session session = _contextMgr.SessionRepository.Select().Where(t => t.ID == id).FirstOrDefault();
            if (session == null) return View("Error", "Track was not found.");

            int trackId = session.TrackID.Value;

            // Delete the conference
            _contextMgr.SessionRepository.Remove(session);

            // Save changes
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Index", new { trackId = trackId });
        }
    }
}
