﻿using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using AutoMapper;
using EventTracker.DAL;
using EventTracker.Helpers;
using EventTracker.Infrastructure;
using EventTracker.Models;

namespace EventTracker.Controllers
{
    
    public class TracksController : Controller
    {
        readonly IContextManager _contextMgr;

        public TracksController(IContextManager contextMgr)
        {
            _contextMgr = contextMgr;
        }

        //
        // GET: /Track/
        public ActionResult Index(int? conferenceId)
        {
            IEnumerable<TrackModel> model;

            if (!conferenceId.HasValue)
            {
                ViewData["Title"] = "Tracks for All Conferences";

                // return ALL tracks for ALL conferences
                model = _contextMgr.TrackRepository.Select()
                                        .ToList()
                                        .Select(track => Mapper.Map<Track, TrackModel>(track));
            }
            else
            {
                // return tracks specific to the specified conference
                Conference conference = _contextMgr.ConferenceRepository.Select()
                    .Where(c => c.ID == conferenceId.Value).FirstOrDefault();

                if (conference == null) return View("Error", "Conference was not found.");

                ViewData["IsOrganizer"] = User.IsOrganizerOfConference(conference);
                ViewData["Title"] = conference.Name;
                ViewData["ConferenceID"] = conference.ID;

                model = _contextMgr.TrackRepository.Select()
                        .Where(t => t.Conference == conference)
                        .ToList()
                        .Select(track => Mapper.Map<Track, TrackModel>(track));
            }

            return View(model);
        }
      
        public ActionResult _PartialListing(int? conferenceId)
        {
            // return tracks specific to the specified conference
            Conference conference = _contextMgr.ConferenceRepository.Select()
                .Where(c => c.ID == conferenceId.Value).FirstOrDefault();

            if (conference == null) return View("PartialError", "Conference was not found.");

            IEnumerable<TrackModel> model = _contextMgr.TrackRepository.Select()
                    .Where(t => t.Conference == conference)
                    .OrderBy(t => t.Title)
                    .ToList()
                    .Select(track => Mapper.Map<Track, TrackModel>(track));

            return View(model);
        }

        [Authorize(Roles=RoleConstants.Organizer)]
        public ActionResult Create(int conferenceId)
        {
            // return tracks specific to the specified conference
            Conference conference = _contextMgr.ConferenceRepository.Select()
                .Where(c => c.ID == conferenceId).FirstOrDefault();

            if (conference == null) return View("Error", "Conference was not found.");

            // Determine if user is allowed to edit this conference
            if (!User.IsOrganizerOfConference(conference))
                return View("Error", "You are not allowed to edit this conference.");

            return View(new TrackModel());
        }

        [HttpPost]
        public ActionResult Create(int conferenceId, TrackModel model, FormCollection collection)
        {
            Conference conference = _contextMgr.ConferenceRepository.Select().Where(c => c.ID == conferenceId).FirstOrDefault();
            if (conference == null) return View("Error", "Conference not found.");

            conference.Tracks.Add(new Track()
                                    {
                                        Conference = conference,
                                        Title = model.Title,
                                        Description = model.Description
                                    });

            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Index", new { conferenceId = conferenceId });
        }

        [Authorize(Roles=RoleConstants.Organizer)]
        public ActionResult Edit(int Id)
        {
            Track track = _contextMgr.TrackRepository.Select().Where(t => t.ID == Id).FirstOrDefault();
            if (track == null) return View("Error", "Track was not found");

            TrackModel model = Mapper.Map<Track, TrackModel>(track);

            return View(model);
        }

        [Authorize(Roles=RoleConstants.Organizer)]
        [HttpPost]
        public ActionResult Edit(TrackModel model, FormCollection collection)
        {
            Track track = _contextMgr.TrackRepository.Select().Where(t => t.ID == model.Id).FirstOrDefault();
            if (track == null) return View("Error", "Track was not found");

            Mapper.Map<TrackModel, Track>(model, track);
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Index", new { conferenceId = track.ConferenceID.Value });
        }

        [Authorize(Roles = "Organizer")]
        public ActionResult Delete(int id)
        {
            Track track = _contextMgr.TrackRepository.Select().Where(t => t.ID == id).FirstOrDefault();
            if (track == null) return View("Error", "Track was not found.");

            int conferenceId = track.ConferenceID.Value;

            // Delete the conference
            _contextMgr.TrackRepository.Remove(track);

            // Save changes
            _contextMgr.Context.SaveChanges();

            return RedirectToAction("Index", new { conferenceId = conferenceId });
        }
    }
}
